# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the hourly weather
conditions. It has one class for this purpose.
"""


class FIOHourly(object):
    """
    This class recieves an ForecastIO object and holds the hourly weather
    conditions. It has one class for this purpose.
    """

    hourly = None

    def __init__(self, forecast_io):
        """
        Recieves an ForecastIO object and gets the hourly weather conditions
        if they are available in the object.
        """
        if forecast_io.has_hourly():
            self.hourly = forecast_io.get_hourly()
            for item in forecast_io.get_hourly().keys():
                setattr(self, item, forecast_io.get_hourly()[item])
            for hour in range(0, self.hours()):
                for item in self.get_hour(hour).keys():
                    setattr(self, 'hour_'+str(hour)+'_'+item, \
                    self.get_hour(hour)[item])

    def get(self, hour=None):
        """
        Returns a dictionary with hourly weather conditions.
        Returns None is none are available.
        A day can be passed as an argument, is so function will call get_hour()
        to return that day.
        Look on function get_hour()
        """
        if hour is None:
            return self.hourly
        else:
            return self.get_hour(hour)

    def get_hour(self, hour):
        """
        Recieves a hour as an argument and returns the prediction for that hour
        if is available. If not, function will return None.
        """
        if hour > self.hours():
            return None
        else:
            return self.get()['data'][hour]

    def hours(self):
        """
        Returns how many hours of prediction are available
        """
        return len(self.get()['data'])



# -*- coding: utf-8 -*-
"""
This module recieves the api key and the configurations to build the request
url.
It then gets the weather data based on those configurations.
The resulting object is used by the other classes to get the information.
"""

from __future__ import print_function
import sys
import json
import requests



class ForecastIO(object):
    """
    This class recieves the api key and the configurations to build the request
    url.
    It then gets the weather data based on those configurations.
    The resulting object is used by the other classes to get the information.
    """

    # pylint: disable=too-many-instance-attributes
    # Many attributes needed to hold all the data
 
    # pylint: disable=too-many-arguments
    # Many arguments needed to build the url

    _darksky_url = 'https://api.darksky.net/forecast/'

    _allowed_excludes_extends = ('currently', 'minutely', 'hourly', \
    'daily', 'alerts', 'flags')

    UNITS_US = 'us'
    UNITS_SI = 'si'
    UNITS_CA = 'ca'
    UNITS_UK = 'uk'
    UNITS_AUTO = 'auto'
    LANG_BOSNIAN = 'bs'
    LANG_GERMAN = 'de'
    LANG_ENGLISH = 'en'
    LANG_SPANISH = 'es'
    LANG_FRENCH = 'fr'
    LANG_ITALIAN = 'it'
    LANG_DUTCH = 'nl'
    LANG_POLISH = 'pl'
    LANG_PORTUGUESE = 'pt'
    LANG_TETUM = 'tet'
    LANG_PIG_LATIN = 'x-pig-latin'
    LANG_RUSSIAN = 'ru'

    def __init__(self, apikey, extend=None, exclude=None, units=UNITS_AUTO, \
    lang=LANG_ENGLISH, time=None, latitude=None, longitude=None):
        """
        A valid api key must be provided in the object instantiation.
        Other options are available.
        Units, language, extended reply can be set.
        It is useful to provide coordinates (latitude and longitude) to get a
        reply.
        """

        if len(apikey) == 32:
            self.forecast_io_api_key = apikey
            self.extend_url = extend
            self.exclude_url = exclude
            self.units_url = units
            self.lang_url = lang
            self.time_url = time
            self.latitude = latitude
            self.longitude = longitude
            if latitude is not None and longitude is not None:
                self.get_forecast(latitude, longitude)
            else:
                print('Latitude or longitude not set')
        else:
            raise ValueError('The API Key doesn\'t seem to be valid.')

    def get_forecast(self, latitude, longitude):
        """
        Gets the weather data and stores it in the respective dictionaries if
        available.
        This function should be used to fetch weather information.
        """
        reply = self.http_get(self.url_builder(latitude, longitude))
        self.forecast = json.loads(reply)

        for item in self.forecast.keys():
            setattr(self, item, self.forecast[item])

    def url_builder(self, latitude, longitude):
        """
        This function is used to build the correct url to make the request
        to the forecast.io api.
        Recieves the latitude and the longitude.
        Return a string with the url.
        """
        try:
            float(latitude)
            float(longitude)
        except ValueError:
            raise ValueError('Latitude (%s) and Longitude (%s) must be a float number' % (latitude, longitude))
        url = self._darksky_url + self.forecast_io_api_key + '/'
        url += str(latitude).strip() + ',' + str(longitude).strip()
        if self.time_url and not self.time_url.isspace():
            url += ',' + self.time_url.strip()
        url += '?units=' + self.units_url.strip()
        url += '&lang=' + self.lang_url.strip()
        if self.exclude_url is not None:
            excludes = ''
            for item in self.exclude_url:
                if item in self._allowed_excludes_extends:
                    excludes += item + ','
            if len(excludes) > 0:
                url += '&exclude=' + excludes.rstrip(',')
        if self.extend_url is not None:
            extends = ''
            for item in self.extend_url:
                if item in self._allowed_excludes_extends:
                    extends += item + ','
            if len(extends) > 0:
                url += '&extend=' + extends.rstrip(',')
        return url
 
    def get_url(self):
        """
        Return the url built from the url_builder() function.

        Returns:
            The built url string
        """
        return self.url_builder(self.latitude, self.longitude)

    def http_get(self, request_url):
        """
        This function recieves the request url and it is used internally to get
        the information via http.
        Returns the response content.
        Raises Timeout, TooManyRedirects, RequestException.
        Raises KeyError if headers are not present.
        Raises HTTPError if responde code is not 200.
        """
        try:
            headers = {'Accept-Encoding': 'gzip, deflate'}
            response = requests.get(request_url, headers=headers)
        except requests.exceptions.Timeout as ext:
            print('Error: Timeout', ext)
        except requests.exceptions.TooManyRedirects as extmr:
            print('Error: TooManyRedirects', extmr)
        except requests.exceptions.RequestException as ex:
            print('Error: RequestException', ex)
            sys.exit(1)

        try:
            self.cache_control = response.headers['Cache-Control']
        except KeyError as kerr:
            print('Warning: Could not get headers. %s' % kerr)
            self.cache_control = None
        try:
            self.expires = response.headers['Expires']
        except KeyError as kerr:
            print('Warning: Could not get headers. %s' % kerr)
            self.extend_url = None
        try:
            self.x_forecast_api_calls = response.headers['X-Forecast-API-Calls']
        except KeyError as kerr:
            print('Warning: Could not get headers. %s' % kerr)
            self.x_forecast_api_calls = None
        try:
            self.x_responde_time = response.headers['X-Response-Time']
        except KeyError as kerr:
            print('Warning: Could not get headers. %s' % kerr)
            self.x_responde_time = None

        if response.status_code is not 200:
            raise requests.exceptions.HTTPError('Bad response, status code: %x' % (response.status_code))

        self.raw_response = response.text
        return self.raw_response

    def has_currently(self):
        """
        Return True if currently information is available. False otherwise.
        """
        return 'currently' in self.forecast

    def get_currently(self):
        """
        Returns currently information or None if it is not available.
        """
        if self.has_currently() == True:
            return self.currently
        else:
            return None

    def has_daily(self):
        """
        Return True if daily information is available. False otherwise.
        """
        return 'daily' in self.forecast

    def get_daily(self):
        """
        Returns daily information or None if it is not available.
        """
        if self.has_daily() == True:
            return self.daily
        else:
            return None

    def has_hourly(self):
        """
        Return True if hourly information is available. False otherwise.
        """
        return 'hourly' in self.forecast

    def get_hourly(self):
        """
        Returns hourly information or None if it is not available.
        """
        if self.has_hourly() == True:
            return self.hourly
        else:
            return None

    def has_minutely(self):
        """
        Return True if minutly information is available. False otherwise.
        """
        return 'minutely' in self.forecast

    def get_minutely(self):
        """
        Returns minutely information or None if it is not available.
        """
        if self.has_minutely() == True:
            return self.minutely
        else:
            return None

    def has_flags(self):
        """
        Return True if flags information is available. False otherwise.
        """
        return 'flags' in self.forecast

    def get_flags(self):
        """
        Returns flags information or None if it is not available.
        """
        if self.has_flags() == True:
            return self.flags
        else:
            return None

    def has_alerts(self):
        """
        Return True if alerts information is available. False otherwise.
        """
        return 'alerts' in self.forecast

    def get_alerts(self):
        """
        Returns alerts information or None if it is not available.
        """
        if self.has_alerts() == True:
            return self.alerts
        else:
            return None



# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the currently weather
conditions. It has one class for this purpose.
"""


class FIOCurrently(object):
    """
    This class recieves an ForecastIO object and holds the currently weather
    conditions.
    """

    currently = None

    def __init__(self, forecast_io):
        """
        Construct a new 'FIOCurrently' object.
        Recieves an ForecastIO object and gets the currently weather conditions
        if they are available in the object.

        Args:
            forecast_io (ForecastIO): The ForecastIO object
        """
        if forecast_io.has_currently():
            self.currently = forecast_io.get_currently()
            for item in self.currently.keys():
                setattr(self, item, self.currently[item])

    def get(self):
        """
        Returns a dictionary with current weather conditions.
        Returns None is none are available.

        Returns:
            Dictionary with current weather conditions.
            None is none are available.
        """
        return self.currently



# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the falgs weather
data. It has one class for this purpose.
"""


class FIOFlags(object):
    """
    This class recieves an ForecastIO object and holds the flags weather
    data. It has one class for this purpose.
    """

    flags = None

    def __init__(self, forecast_io):
        """
        Recieves an ForecastIO object and gets the flags weather data
        if they are available in the object.
        """
        if forecast_io.has_flags():
            self.flags = forecast_io.get_flags()
            for item in self.flags.keys():
                setattr(self, item, self.flags[item])

    def get(self):
        """
        Returns a dictionary with flags weather data
        """
        return self.flags

    def available_flags(self):
        """
        Returns the available flags available in weather data
        """
        return self.get().keys()



# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the minutely weather
conditions. It has one class for this purpose.
"""

class FIOMinutely(object):
    """
    This class recieves an ForecastIO object and holds the minutely weather
    conditions. It has one class for this purpose.
    """

    minutely = None

    def __init__(self, forecast_io):
        """
        Recieves an ForecastIO object and gets the minutely weather conditions
        if they are available in the object.
        """
        if forecast_io.has_minutely():
            self.minutely = forecast_io.get_minutely()
            for item in forecast_io.get_minutely().keys():
                setattr(self, item, forecast_io.get_minutely()[item])
            for minute in range(0, self.minutes()):
                for item in self.get_minute(minute).keys():
                    setattr(self, 'minute_'+str(minute)+'_'+item, \
                    self.get_minute(minute)[item])

    def get(self, minute=None):
        """
        Returns a dictionary with minutely weather conditions.
        Returns None is none are available.
        A day can be passed as an argument, is so function will call get_minute()
        to return that day.
        Look on function get_minute()
        """
        if minute is None:
            return self.minutely
        else:
            return self.get_minute(minute)

    def get_minute(self, minute):
        """
        Recieves a minute as an argument and returns the prediction for that
        minute if is available. If not, function will return None.
        """
        if minute > self.minutes():
            return None
        else:
            return self.get()['data'][minute]

    def minutes(self):
        """
        Returns how many minutes of prediction are available
        """
        return len(self.get()['data'])



# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the alerts weather
conditions. It has one class for this purpose.
"""


class FIOAlerts(object):
    """
    This class recieves an ForecastIO object and holds the alerts weather
    conditions. It has one class for this purpose.
    """
    alerts = None

    def __init__(self, forecast_io):
        """
        Recieves an ForecastIO object and gets the alerts weather conditions
        if they are available in the object.
        """
        if forecast_io.has_alerts():
            self.alerts = forecast_io.get_alerts()

    def get(self, alert=None):
        """
        Returns a dictionary with alert weather conditions.
        Returns None is none are available.
        A day can be passed as an argument, is so function will call get_alert()
        to return that day.
        Look on function get_alert()
        """
        if alert is None:
            return self.alerts
        else:
            return self.get_alert(alert)

    def get_alert(self, alert):
        """
        Recieves a day as an argument and returns the prediction for that alert
        if is available. If not, function will return None.
        """
        if alert > self.alerts_count() or self.alerts_count() is None:
            return None
        else:
            return self.get()[alert-1]

    def alerts_count(self):
        """
        Returns how many alerts of prediction are available.
        Returns None if none is available
        """
        if self.get() is None:
            return None
        else:
            return len(self.get())



# -*- coding: utf-8 -*-
"""
This module recieves an ForecastIO object and holds the daily weather
conditions. It has one class for this purpose.
"""

class FIODaily(object):
    """
    This class recieves an ForecastIO object and holds the daily weather
    conditions. It has one class for this purpose.
    """

    daily = None

    def __init__(self, forecast_io):
        """
        Recieves an ForecastIO object and gets the daily weather conditions
        if they are available in the object.
        """
        if forecast_io.has_daily():
            self.daily = forecast_io.get_daily()
            for item in forecast_io.get_daily().keys():
                setattr(self, item, forecast_io.get_daily()[item])
            for day in range(0, self.days()):
                for item in self.get_day(day).keys():
                    setattr(self, 'day_'+str(day)+'_'+item, \
                    self.get_day(day)[item])

    def get(self, day=None):
        """
        Returns a dictionary with daily weather conditions.
        Returns None is none are available.
        A day can be passed as an argument, if so function will call get_day()
        to return that day.
        Look on function get_day()
        """
        if day is None:
            return self.daily
        else:
            return self.get_day(day)

    def get_day(self, day):
        """
        Recieves a day as an argument and returns the prediction for that day if
        is available. If not, function will return None.
        """
        if day > self.days():
            return None
        else:
            return self.get()['data'][day]

    def days(self):
        """
        Returns how many days of prediction are available
        """
        return len(self.get()['data'])



# -*- coding: utf-8 -*-
"""
__init__.py file for forecastiopy module.
Defines the __all__ modules.
"""

__all__ = ['ForecastIO', 'FIOCurrently', 'FIOMinutely', \
           'FIOHourly', 'FIODaily', 'FIOFlags', 'FIOAlerts']



from setuptools import setup



setup(name='forecastiopy',
      version='0.22',
      description='A wrapper to access weather data provided by darksky.net',
      url='http://github.com/dvdme/forecastiopy',
      download_url='http://github.com/dvdme/forecastiopy/tarball/0.2',
      author='David Ervideira',
      author_email='david.dme@gmail.com',
      license='Eclipse Public License',
      packages=['forecastiopy'],
      keywords=['weather','forecastio', 'forecast.io','rest', 'darksky','darksky.net'],
      install_requires=[
          'requests',
      ],
      zip_safe=True)